@@ -1,4 +1,4 @@
-__version__ = "0.17.0"
+__version__ = "0.17.1"
 
 from e3nn_jax._src.config import config
 from e3nn_jax._src.rotation import (
@@ -52,7 +52,7 @@ from e3nn_jax._src.su2 import su2_clebsch_gordan, su2_generators
 from e3nn_jax._src.so3 import clebsch_gordan, generators
 from e3nn_jax._src.instruction import Instruction
 from e3nn_jax._src.irreps import Irrep, MulIrrep, Irreps
-from e3nn_jax._src.irreps_array import IrrepsArray, concatenate, stack, mean, norm, normal
+from e3nn_jax._src.irreps_array import IrrepsArray, concatenate, stack, mean, norm, normal, dot
 from e3nn_jax._src.irreps_array import sum_ as sum
 from e3nn_jax._src.spherical_harmonics import spherical_harmonics, sh, legendre
 from e3nn_jax._src.radial import sus, soft_one_hot_linspace, bessel, poly_envelope, soft_envelope
@@ -75,6 +75,7 @@ from e3nn_jax._src.reduced_tensor_product import (
     reduced_antisymmetric_tensor_product_basis,
 )
 from e3nn_jax._src.s2grid import s2_irreps, to_s2grid, to_s2point, from_s2grid, s2_dirac, SphericalSignal
+from e3nn_jax._src.tensor_product_with_spherical_harmonics import tensor_product_with_spherical_harmonics
 
 # make submodules flax and haiku available
 from e3nn_jax import flax, haiku
@@ -140,6 +141,7 @@ __all__ = [
     "mean",
     "norm",
     "normal",
+    "dot",
     "sum",
     "spherical_harmonics",
     "sh",
@@ -172,6 +174,7 @@ __all__ = [
     "from_s2grid",
     "s2_dirac",
     "SphericalSignal",
+    "tensor_product_with_spherical_harmonics",
     "flax",
     "haiku",
 ]
