@@ -1,33 +1,4 @@
-r"""Transformation between two representations of a signal on the sphere.
-
-.. math:: f: S^2 \longrightarrow \mathbb{R}
-
-is a signal on the sphere.
-
-It can be decomposed into the basis of the spherical harmonics:
-
-.. math:: f(x) = \sum_{l=0}^{l_{\mathit{max}}} F^l \cdot Y^l(x)
-
-it is made of :math:`(l_{\mathit{max}} + 1)^2` real numbers represented in the above formula by the familly of vectors
-:math:`F^l \in \mathbb{R}^{2l+1}`.
-
-Another representation is the discretization around the sphere. For this representation we chose a particular grid of size
-:math:`(N, M)`
-
-.. math::
-
-    x_{ij} &= (\sin(\beta_i) \sin(\alpha_j), \cos(\beta_i), \sin(\beta_i) \cos(\alpha_j))
-
-    \beta_i &= \pi (i + 0.5) / N
-
-    \alpha_j &= 2 \pi j / M
-
-In the code, :math:`N` is called ``res_beta`` and :math:`M` is ``res_alpha``.
-
-The discrete representation is therefore
-
-.. math:: \{ h_{ij} = f(x_{ij}) \}_{ij}
-"""
+import math
 from typing import Callable, List, Optional, Tuple, Union
 
 import jax
@@ -52,16 +23,76 @@ class SphericalSignal:
         p_arg: parity of the argument of the signal, either ``+1`` or ``-1``
 
     Examples:
-        >>> coeffs = e3nn.IrrepsArray("0e + 1o", jnp.array([1, 2, 0, 0.0]))
-        >>> signal = e3nn.to_s2grid(coeffs, 50, 49, quadrature="soft")
-        >>> signal.integrate()
-        1x0e [12.566371]
 
-        >>> jnp.set_printoptions(precision=2, suppress=True)
-        >>> signal = signal.apply(jnp.exp)
-        >>> coeffs = e3nn.from_s2grid(signal, e3nn.s2_irreps(2))
-        >>> coeffs
-        1x0e+1x1o+1x2e [12.52 15.47 -0.    0.    0.    0.   -5.35  0.   -9.27]
+        .. jupyter-execute::
+            :hide-code:
+
+            import e3nn_jax as e3nn
+            import jax
+            import jax.numpy as jnp
+            jnp.set_printoptions(precision=3, suppress=True)
+
+        Create a null signal:
+
+        .. jupyter-execute::
+
+            e3nn.SphericalSignal.zeros(50, 49, quadrature="soft")
+
+        Create a signal from a spherical harmonic expansion:
+
+        .. jupyter-execute::
+
+            coeffs = e3nn.IrrepsArray("0e + 1o", jnp.array([1.0, 0.0, 2.0, 0.0]))
+            signal = e3nn.to_s2grid(coeffs, 50, 49, quadrature="soft")
+            signal
+
+        Apply a function to the signal:
+
+        .. jupyter-execute::
+
+            signal = signal.apply(jnp.exp)
+            signal
+
+        Convert the signal back to a spherical harmonic expansion:
+
+        .. jupyter-execute::
+
+            irreps = e3nn.s2_irreps(4)
+            coeffs = e3nn.from_s2grid(signal, irreps)
+            coeffs["4e"]
+
+        Resample the signal to a different grid resolution:
+
+        .. jupyter-execute::
+
+            signal = signal.resample(100, 99, lmax=5)
+            signal
+
+        Compute the integral of the signal:
+
+        .. jupyter-execute::
+
+            signal.integrate()
+
+        Rotate the signal (we need to determine ``lmax`` because the rotation is done in the Fourier domain):
+
+        .. jupyter-execute::
+
+            signal = signal.transform_by_angles(jnp.pi / 2, jnp.pi / 3, 0.0, lmax=5)
+
+        Sample a point on the sphere, using the signal as a density function:
+
+        .. jupyter-execute::
+
+            indices = signal.sample(jax.random.PRNGKey(0))
+            signal.grid_vectors[indices], signal.grid_values[indices]
+
+        Plot the signal:
+
+        .. jupyter-execute::
+
+            import plotly.graph_objects as go
+            go.Figure([go.Surface(signal.plotly_surface())])
     """
     grid_values: jnp.ndarray
     quadrature: str
@@ -89,6 +120,13 @@ class SphericalSignal:
         self.p_val = p_val
         self.p_arg = p_arg
 
+    @staticmethod
+    def zeros(
+        res_beta: int, res_alpha: int, quadrature: str, *, p_val: int = 1, p_arg: int = -1, dtype: jnp.dtype = jnp.float32
+    ) -> "SphericalSignal":
+        """Create a null signal on a grid."""
+        return SphericalSignal(jnp.zeros((res_beta, res_alpha), dtype), quadrature, p_val=p_val, p_arg=p_arg)
+
     def __repr__(self) -> str:
         return (
             "SphericalSignal("
@@ -98,15 +136,24 @@ class SphericalSignal:
 
     def __mul__(self, scalar: float) -> "SphericalSignal":
         """Multiply SphericalSignal by a scalar."""
-        return SphericalSignal(self.grid_values * scalar, self.quadrature, self.p_val, self.p_arg)
+        if isinstance(scalar, SphericalSignal):
+            raise ValueError("Multiplication of two SphericalSignals is not supported.")
+
+        if isinstance(scalar, e3nn.IrrepsArray):
+            if scalar.irreps != e3nn.Irreps("0e"):
+                raise ValueError("Scalar must be a 0e IrrepsArray.")
+            scalar = scalar.array[..., 0]
+
+        scalar = jnp.asarray(scalar)[..., None, None]
+        return SphericalSignal(self.grid_values * scalar, self.quadrature, p_val=self.p_val, p_arg=self.p_arg)
 
     def __rmul__(self, scalar: float) -> "SphericalSignal":
         """Multiply SphericalSignal by a scalar."""
-        return SphericalSignal(self.grid_values * scalar, self.quadrature, self.p_val, self.p_arg)
+        return self * scalar
 
     def __truediv__(self, scalar: float) -> "SphericalSignal":
         """Divide SphericalSignal by a scalar."""
-        return SphericalSignal(self.grid_values / scalar, self.quadrature, self.p_val, self.p_arg)
+        return self * (1 / scalar)
 
     def __add__(self, other: "SphericalSignal") -> "SphericalSignal":
         """Add to another SphericalSignal."""
@@ -120,21 +167,15 @@ class SphericalSignal:
         if self.quadrature != other.quadrature:
             raise ValueError("Quadrature for both signals must be identical.")
 
-        return SphericalSignal(self.grid_values + other.grid_values, self.quadrature, self.p_val, self.p_arg)
+        return SphericalSignal(self.grid_values + other.grid_values, self.quadrature, p_val=self.p_val, p_arg=self.p_arg)
 
     def __sub__(self, other: "SphericalSignal") -> "SphericalSignal":
         """Subtract another SphericalSignal."""
-        if self.grid_resolution != other.grid_resolution:
-            raise ValueError(
-                "Grid resolutions for both signals must be identical. "
-                "Use .resample() to change one of the grid resolutions."
-            )
-        if (self.p_val, self.p_arg) != (other.p_val, other.p_arg):
-            raise ValueError("Parity for both signals must be identical.")
-        if self.quadrature != other.quadrature:
-            raise ValueError("Quadrature for both signals must be identical.")
+        return self + (-other)
 
-        return SphericalSignal(self.grid_values - other.grid_values, self.quadrature, self.p_val, self.p_arg)
+    def __neg__(self) -> "SphericalSignal":
+        """Negate SphericalSignal."""
+        return SphericalSignal(-self.grid_values, self.quadrature, p_val=self.p_val, p_arg=self.p_arg)
 
     @property
     def shape(self) -> Tuple[int, ...]:
@@ -146,6 +187,11 @@ class SphericalSignal:
         """Returns the dtype of this signal."""
         return self.grid_values.dtype
 
+    @property
+    def ndim(self) -> int:
+        """Returns the number of dimensions of this signal."""
+        return self.grid_values.ndim
+
     @property
     def grid_y(self) -> jnp.ndarray:
         """Returns y-values on the grid for this signal."""
@@ -160,7 +206,7 @@ class SphericalSignal:
 
     @property
     def grid_vectors(self) -> jnp.ndarray:
-        """Returns the coordinates of the points on the sphere."""
+        """Returns the coordinates of the points on the sphere. Shape: ``(res_beta, res_alpha, 3)``."""
         y, alpha, _ = _s2grid(self.res_beta, self.res_alpha, self.quadrature)
         return _s2grid_vectors(y, alpha)
 
@@ -263,6 +309,7 @@ class SphericalSignal:
 
         Currently cannot be wrapped with jax.jit().
         """
+        # TODO: Still has the bug `ValueError: buffer source array is read-only`
         grid_resolution = self.grid_resolution
         x1, f1 = self.grid_vectors, self.grid_values
         x1, f1 = jax.tree_map(lambda arr: np.asarray(arr.copy()), (x1, f1))
@@ -302,8 +349,8 @@ class SphericalSignal:
             scale_radius_by_amplitude (bool): to rescale the output vectors with the amplitude of the signal
 
         Returns:
-            r (jnp.ndarray): vectors on the sphere, shape ``(res_beta + 2, res_alpha + 1, 3)``
-            f (jnp.ndarray): padded signal, shape ``(res_beta + 2, res_alpha + 1)``
+            r (`jax.numpy.ndarray`): vectors on the sphere, shape ``(res_beta + 2, res_alpha + 1, 3)``
+            f (`jax.numpy.ndarray`): padded signal, shape ``(res_beta + 2, res_alpha + 1)``
         """
         f, y, alpha = self.grid_values, self.grid_y, self.grid_alpha
 
@@ -338,17 +385,24 @@ class SphericalSignal:
         Returns:
             dict: dictionary that can be plotted with plotly
 
-        Example:
+        Examples:
 
         .. jupyter-execute::
 
             import jax.numpy as jnp
             import e3nn_jax as e3nn
-            coeffs = e3nn.IrrepsArray("0e + 1o", jnp.array([1, 2, 0, 0.0]))
-            signal = e3nn.to_s2grid(coeffs, 50, 69, quadrature="gausslegendre")
+            coeffs = e3nn.normal(e3nn.s2_irreps(5), jax.random.PRNGKey(0))
+            signal = e3nn.to_s2grid(coeffs, 70, 141, quadrature="gausslegendre")
 
             import plotly.graph_objects as go
             go.Figure([go.Surface(signal.plotly_surface())])
+
+        One can also scale the radius of the sphere by the amplitude of the signal:
+
+        .. jupyter-execute::
+
+            go.Figure([go.Surface(signal.plotly_surface(scale_radius_by_amplitude=True))])
+
         """
         r, f = self.pad_to_plot(translation=translation, scale_radius_by_amplitude=scale_radius_by_amplitude)
         return dict(
@@ -364,7 +418,7 @@ class SphericalSignal:
         The integral of a constant signal of value 1 is 4pi.
 
         Returns:
-            e3nn.IrrepsArray: integral of the signal
+            `IrrepsArray`: integral of the signal
         """
         values = self.quadrature_weights[..., None] * self.grid_values
         values = jnp.sum(values, axis=-2)
@@ -373,6 +427,57 @@ class SphericalSignal:
         integral_irreps = {1: "0e", -1: "0o"}[self.p_val]
         return e3nn.IrrepsArray(integral_irreps, values)
 
+    def sample(self, key: jnp.ndarray) -> Tuple[jnp.ndarray, jnp.ndarray]:
+        """Sample a point on the sphere using the signal as a probability distribution.
+
+        The probability distribution does not need to be normalized.
+
+        Args:
+            key (`jax.numpy.ndarray`): random key
+
+        Returns:
+            (tuple): tuple containing:
+
+                beta_index (`jax.numpy.ndarray`): index of the sampled beta
+                alpha_index (`jax.numpy.ndarray`): index of the sampled alpha
+
+        Examples:
+
+        .. jupyter-execute::
+            :hide-code:
+
+            import jax
+            import jax.numpy as jnp
+            import e3nn_jax as e3nn
+
+
+        .. jupyter-execute::
+
+            coeffs = e3nn.IrrepsArray("0e + 1o", jnp.array([1.0, 2.0, 0.0, 0.0]))
+            signal = e3nn.to_s2grid(coeffs, 50, 69, quadrature="gausslegendre")
+            signal = signal.apply(jnp.exp)
+
+            beta_index, alpha_index = signal.sample(jax.random.PRNGKey(0))
+            print(beta_index, alpha_index)
+            print(signal.grid_vectors[beta_index, alpha_index])
+        """
+
+        def f(k, p_ya):  # single signal only
+            assert k.shape == (2,)
+            assert p_ya.shape == (self.res_beta, self.res_alpha)
+            k1, k2 = jax.random.split(k)
+            p_y = self.quadrature_weights * jnp.sum(p_ya, axis=1)  # [y]
+            y_index = jax.random.choice(k1, jnp.arange(self.res_beta), p=p_y)  # []
+            alpha_index = jax.random.choice(k2, jnp.arange(self.res_alpha), p=p_ya[y_index])  # []
+            return y_index, alpha_index
+
+        vf = f
+        for _ in range(self.ndim - 2):
+            vf = jax.vmap(vf)
+
+        keys = jax.random.split(key, math.prod(self.shape[:-2])).reshape(self.shape[:-2] + key.shape)
+        return vf(keys, self.grid_values)
+
 
 jax.tree_util.register_pytree_node(
     SphericalSignal,
@@ -383,16 +488,88 @@ jax.tree_util.register_pytree_node(
 )
 
 
-def s2_sum_of_diracs(positions: jnp.ndarray, weights: jnp.ndarray, lmax: int, *, p_val: int, p_arg: int) -> e3nn.IrrepsArray:
-    r"""Spherical harmonics coefficients of the sum of (almost-)Dirac deltas on the sphere.
+def s2_dirac(position: jnp.ndarray, lmax: int, *, p_val: int, p_arg: int) -> e3nn.IrrepsArray:
+    r"""Spherical harmonics expansion of a Dirac delta on the sphere.
 
-    The integral of each Dirac delta is 1.
+    The integral of the Dirac delta is 1.
+
+    Args:
+        position (`jax.numpy.ndarray`): position of the delta, shape ``(3,)``.
+            It will be normalized to have a norm of 1.
+
+        lmax (int): maximum degree of the spherical harmonics expansion
+        p_val (int): parity of the value of the signal on the sphere (1 or -1)
+        p_arg (int): parity of the argument of the signal on the sphere (1 or -1)
+
+    Returns:
+        `IrrepsArray`: Spherical harmonics coefficients
+
+    Examples:
+
+    .. jupyter-execute::
+        :hide-code:
+
+        import jax.numpy as jnp
+        import e3nn_jax as e3nn
+        import plotly.graph_objects as go
+
+    .. jupyter-execute::
+
+        position = jnp.array([0.0, 0.0, 1.0])
+
+        coeffs_3 = e3nn.s2_dirac(position, 3, p_val=1, p_arg=-1)
+        coeffs_6 = e3nn.s2_dirac(position, 6, p_val=1, p_arg=-1)
+        coeffs_9 = e3nn.s2_dirac(position, 9, p_val=1, p_arg=-1)
+
+    .. jupyter-execute::
+        :hide-code:
+
+        signal_3 = e3nn.to_s2grid(coeffs_3, 50, 69, quadrature="gausslegendre")
+        signal_6 = e3nn.to_s2grid(coeffs_6, 50, 69, quadrature="gausslegendre")
+        signal_9 = e3nn.to_s2grid(coeffs_9, 50, 69, quadrature="gausslegendre")
+
+        axis = dict(
+            # showbackground=False,
+            # showgrid=False,
+            # showline=False,
+            showticklabels=False,
+            # ticks="",
+            title="",
+        )
+        go.Figure(
+            [
+                go.Surface(dict(**signal_3.plotly_surface(jnp.array([-2.1, 0, 0])), showscale=False)),
+                go.Surface(dict(**signal_6.plotly_surface(), showscale=False)),
+                go.Surface(dict(**signal_9.plotly_surface(jnp.array([2.1, 0, 0])), showscale=False)),
+            ],
+            layout=go.Layout(
+                scene=dict(
+                    xaxis=dict(range=[-3.1, 3.1], **axis),
+                    yaxis=dict(range=[-1, 1], **axis),
+                    zaxis=dict(range=[-1, 1], **axis),
+                    camera=dict(
+                        eye=dict(x=0.0, y=1.0, z=3.0),
+                        up=dict(x=0.0, y=1.0, z=0.0),
+                    ),
+                    aspectratio=dict(x=3.1, y=1, z=1),
+                ),
+            ),
+        )
+
+    Note:
+
+        To compute a sum of weighted Dirac deltas, use:
+
+        .. jupyter-execute::
+
+            positions = jnp.array([[0, 0, 0], [1, 0, 0], [0, 1, 0], [0, 0, 1.0]])
+            weights = jnp.array([1, 1, -1, -1.0])
+
+            e3nn.sum(e3nn.s2_dirac(positions, 4, p_val=1, p_arg=-1) * weights[:, None], axis=0)
     """
-    weights = weights[..., None]
-    positions, _ = jnp.broadcast_arrays(positions, weights)
     irreps = s2_irreps(lmax, p_val, p_arg)
-    y = e3nn.spherical_harmonics(irreps, positions, normalize=True, normalization="integral")  # [..., N, dim]
-    return e3nn.sum(y * weights, axis=-2) / jnp.sqrt(4 * jnp.pi)
+    coeffs = e3nn.spherical_harmonics(irreps, position, normalize=True, normalization="integral")  # [dim]
+    return coeffs / jnp.sqrt(4 * jnp.pi)
 
 
 def s2_irreps(lmax: int, p_val: int = 1, p_arg: int = -1) -> e3nn.Irreps:
@@ -423,22 +600,22 @@ def from_s2grid(
     normalization: str = "integral",
     lmax_in: Optional[int] = None,
     fft: bool = True,
-):
+) -> e3nn.IrrepsArray:
     r"""Transform signal on the sphere into spherical harmonics coefficients.
 
     The output has degree :math:`l` between 0 and lmax, and parity :math:`p = p_{val}p_{arg}^l`
 
-    The inverse transformation of :func:`e3nn_jax.to_s2grid`
+    The inverse transformation of :func:`to_s2grid`
 
     Args:
         x (`SphericalSignal`): signal on the sphere of shape ``(..., y/beta, alpha)``
-        irreps (e3nn.Irreps): irreps of the coefficients
+        irreps (`Irreps`): irreps of the coefficients
         normalization ({'norm', 'component', 'integral'}): normalization of the spherical harmonics basis
         lmax_in (int, optional): maximum degree of the input signal, only used for normalization purposes
         fft (bool): True if we use FFT, False if we use the naive implementation
 
     Returns:
-        `e3nn_jax.IrrepsArray`: coefficient array of shape ``(..., (lmax+1)^2)``
+        `IrrepsArray`: coefficient array of shape ``(..., (lmax+1)^2)``
     """
     res_beta, res_alpha = x.grid_resolution
 
@@ -491,10 +668,10 @@ def to_s2grid(
 ) -> SphericalSignal:
     r"""Sample a signal on the sphere given by the coefficient in the spherical harmonics basis.
 
-    The inverse transformation of :func:`e3nn_jax.from_s2grid`
+    The inverse transformation of :func:`from_s2grid`
 
     Args:
-        coeffs (`e3nn_jax.IrrepsArray`): coefficient array
+        coeffs (`IrrepsArray`): coefficient array
         res_beta (int): number of points on the sphere in the :math:`\theta` direction
         res_alpha (int): number of points on the sphere in the :math:`\phi` direction
         normalization ({'norm', 'component', 'integral'}): normalization of the basis
@@ -505,6 +682,57 @@ def to_s2grid(
 
     Returns:
         `SphericalSignal`: signal on the sphere of shape ``(..., y/beta, alpha)``
+
+    Note:
+
+        We use a rectangular grid for the :math:`\beta` and :math:`\alpha` angles.
+        The grid is uniform in the :math:`\alpha` angle while for :math:`\beta`, two different quadratures are available:
+
+        * The `soft <https://link.springer.com/article/10.1007/s00041-008-9013-5>`_
+          quadrature is a uniform sampling of the beta angle.
+        * The `gauss-legendre <https://en.wikipedia.org/wiki/Gauss%E2%80%93Legendre_quadrature>`_
+          quadrature is a quadrature rule that is exact for polynomials of degree ``2 res_beta - 1``.
+          On the sphere it is exact only for polynomials of :math:`y`.
+
+        .. jupyter-execute::
+            :hide-code:
+
+            import jax.numpy as jnp
+            import e3nn_jax as e3nn
+            import plotly.graph_objects as go
+
+            soft = e3nn.SphericalSignal.zeros(10, 19, "soft")
+            gauss = e3nn.SphericalSignal.zeros(10, 19, "gausslegendre")
+
+            axis = dict(showticklabels=False, title="", range=[-1.1, 1.1])
+
+            go.Figure(
+                data=[
+                    go.Scatter3d(
+                        x=s.grid_vectors[:, :, 0].reshape(-1),
+                        y=s.grid_vectors[:, :, 1].reshape(-1),
+                        z=s.grid_vectors[:, :, 2].reshape(-1),
+                        mode="markers",
+                        marker=dict(
+                            color=c,
+                            size=100 * jnp.broadcast_to(s.quadrature_weights[:, None], s.grid_resolution).reshape(-1)
+                        ),
+                        name=s.quadrature,
+                    )
+                    for s, c in zip([soft, gauss], ["blue", "red"])
+                ],
+                layout=go.Layout(
+                    scene=dict(
+                        xaxis=axis,
+                        yaxis=axis,
+                        zaxis=axis,
+                        camera=dict(
+                            eye=dict(x=0.4, y=0.0, z=1.3),
+                            up=dict(x=0.0, y=1.0, z=0.0),
+                        ),
+                    ),
+                ),
+            )
     """
     coeffs = coeffs.regroup()
     lmax = coeffs.irreps.ls[-1]
@@ -552,15 +780,15 @@ def to_s2point(
 ) -> e3nn.IrrepsArray:
     """Evaluate a signal on the sphere given by the coefficient in the spherical harmonics basis.
 
-    It computes the same thing as :func:`e3nn_jax.to_s2grid` but at a single point.
+    It computes the same thing as :func:`to_s2grid` but at a single point.
 
     Args:
-        coeffs (`e3nn_jax.IrrepsArray`): coefficient array of shape ``(*shape1, irreps)``
+        coeffs (`IrrepsArray`): coefficient array of shape ``(*shape1, irreps)``
         point (`jax.numpy.ndarray`): point on the sphere of shape ``(*shape2, 3)``
         normalization ({'norm', 'component', 'integral'}): normalization of the basis
 
     Returns:
-        `jax.numpy.ndarray`: the value of the signal at the point, of shape ``(*shape1, *shape2, irreps)``
+        `IrrepsArray`: signal on the sphere of shape ``(*shape1, *shape2, irreps)``
     """
     coeffs = coeffs.regroup()
 
@@ -637,9 +865,10 @@ def _s2grid(res_beta: int, res_alpha: int, quadrature: str) -> Tuple[np.ndarray,
         quadrature (str): "soft" or "gausslegendre"
 
     Returns:
-        y (`numpy.ndarray`): array of shape ``(res_beta)``
-        alpha (`numpy.ndarray`): array of shape ``(res_alpha)``
-        qw (`numpy.ndarray`): array of shape ``(res_beta)``, ``sum(qw) = 1``
+        (tuple): tuple containing:
+            y (`numpy.ndarray`): array of shape ``(res_beta)``
+            alpha (`numpy.ndarray`): array of shape ``(res_alpha)``
+            qw (`numpy.ndarray`): array of shape ``(res_beta)``, ``sum(qw) = 1``
     """
 
     if quadrature == "soft":
@@ -671,11 +900,12 @@ def _spherical_harmonics_s2grid(lmax: int, res_beta: int, res_alpha: int, *, qua
         quadrature (str): "soft" or "gausslegendre"
 
     Returns:
-        y (`jax.numpy.ndarray`): array of shape ``(res_beta)``
-        alphas (`jax.numpy.ndarray`): array of shape ``(res_alpha)``
-        sh_y (`jax.numpy.ndarray`): array of shape ``(res_beta, (lmax + 1)(lmax + 2)/2)``
-        sh_alpha (`jax.numpy.ndarray`): array of shape ``(res_alpha, 2 * lmax + 1)``
-        qw (`jax.numpy.ndarray`): array of shape ``(res_beta)``
+        (tuple): tuple containing:
+            y (`jax.numpy.ndarray`): array of shape ``(res_beta)``
+            alphas (`jax.numpy.ndarray`): array of shape ``(res_alpha)``
+            sh_y (`jax.numpy.ndarray`): array of shape ``(res_beta, (lmax + 1)(lmax + 2)/2)``
+            sh_alpha (`jax.numpy.ndarray`): array of shape ``(res_alpha, 2 * lmax + 1)``
+            qw (`jax.numpy.ndarray`): array of shape ``(res_beta)``
     """
     y, alphas, qw = _s2grid(res_beta, res_alpha, quadrature)
     y, alphas, qw = jax.tree_util.tree_map(lambda x: jnp.asarray(x, dtype), (y, alphas, qw))
@@ -826,9 +1056,9 @@ def _rollout_sh(m: jnp.ndarray, lmax: int) -> jnp.ndarray:
     """
     Expand spherical harmonic representation.
     Args:
-        m: jnp.ndarray of shape (..., (lmax+1)*(lmax+2)/2)
+        m (`jax.numpy.ndarray`): of shape (..., (lmax+1)*(lmax+2)/2)
     Returns:
-        jnp.ndarray of shape (..., (lmax+1)**2)
+        `jax.numpy.ndarray`: of shape (..., (lmax+1)**2)
     """
     assert m.shape[-1] == (lmax + 1) * (lmax + 2) // 2
     m_full = jnp.zeros((*m.shape[:-1], (lmax + 1) ** 2), dtype=m.dtype)
