@@ -7,6 +7,7 @@ import jax.numpy as jnp
 import jaxlib
 
 import e3nn_jax as e3nn
+from e3nn_jax._src.util.jit import jit_code
 
 
 # https://stackoverflow.com/a/15008806/1008938
@@ -32,6 +33,7 @@ def main():
     parser.add_argument("--extrachannels", type=t_or_f, default=False)
     parser.add_argument("--fused", type=t_or_f, default=False)
     parser.add_argument("--sparse", type=t_or_f, default=False)
+    parser.add_argument("--weights", type=t_or_f, default=True)
     parser.add_argument("--lists", type=t_or_f, default=False)
     parser.add_argument("--module", type=t_or_f, default=False)
     parser.add_argument("-n", type=int, default=1000)
@@ -39,6 +41,11 @@ def main():
 
     args = parser.parse_args()
 
+    args.irreps_in1 = e3nn.Irreps(args.irreps_in1 if args.irreps_in1 else args.irreps)
+    args.irreps_in2 = e3nn.Irreps(args.irreps_in2 if args.irreps_in2 else args.irreps)
+    args.irreps_out = e3nn.Irreps(args.irreps_out if args.irreps_out else args.irreps)
+    del args.irreps
+
     def k():
         k.key, x = jax.random.split(k.key)
         return x
@@ -55,10 +62,6 @@ def main():
         print(f"{key:>18} : {val}")
     print("=" * 40)
 
-    irreps_in1 = e3nn.Irreps(args.irreps_in1 if args.irreps_in1 else args.irreps)
-    irreps_in2 = e3nn.Irreps(args.irreps_in2 if args.irreps_in2 else args.irreps)
-    irreps_out = e3nn.Irreps(args.irreps_out if args.irreps_out else args.irreps)
-
     kwargs = dict(
         custom_einsum_jvp=args.custom_einsum_jvp,
         fused=args.fused,
@@ -70,7 +73,8 @@ def main():
     def tp(x1, x2):
         if args.module:
             assert not args.extrachannels
-            return e3nn.haiku.FullyConnectedTensorProduct(irreps_out)(x1, x2, **kwargs)
+            assert args.weights
+            return e3nn.haiku.FullyConnectedTensorProduct(args.irreps_out)(x1, x2, **kwargs)
         else:
             if args.extrachannels:
                 assert not args.module
@@ -82,12 +86,17 @@ def main():
             else:
                 x = e3nn.tensor_product(x1, x2, **kwargs)
 
-            return e3nn.haiku.Linear(irreps_out)(x)
-
-    inputs = (e3nn.normal(irreps_in1, k(), (args.batch,)), e3nn.normal(irreps_in2, k(), (args.batch,)))
+            if args.weights:
+                return e3nn.haiku.Linear(args.irreps_out)(x)
+            else:
+                return x.filter(keep=args.irreps_out)
 
+    inputs = (e3nn.normal(args.irreps_in1, k(), (args.batch,)), e3nn.normal(args.irreps_in2, k(), (args.batch,)))
     w = tp.init(k(), *inputs)
 
+    # Ensure everything is on the GPU (shouldn't be necessary, but just in case)
+    w, inputs = jax.tree_util.tree_map(jax.device_put, (w, inputs))
+
     print(f"{sum(x.size for x in jax.tree_util.tree_leaves(w))} parameters")
 
     f = tp.apply
@@ -99,6 +108,7 @@ def main():
         f = lambda w, x1, x2: f_1(w, x1, x2).array
 
     if args.backward:
+        assert args.weights
         # tanh() forces it to realize the grad as a full size matrix rather than expanded (stride 0) ones
         f_2 = f
         f = jax.value_and_grad(
@@ -126,19 +136,8 @@ def main():
     print()
     print(f"{1e3 * perloop:.2f} ms")
 
-    c = jax.xla_computation(f)(w, *inputs)
-
-    backend = jax.lib.xla_bridge.get_backend()
-    e = backend.compile(c)
-    import jaxlib.xla_extension as xla_ext
-
-    option = xla_ext.HloPrintOptions.fingerprint()
-    option.print_operand_shape = False
-    option.print_result_shape = False
-    option.print_program_shape = True
-
-    with open("xla.txt", "wt") as f:
-        f.write(e.hlo_modules()[0].to_string(option))
+    with open("xla.txt", "wt") as file:
+        file.write(jit_code(f, w, *inputs))
 
 
 if __name__ == "__main__":
